import * as React from 'react';
import { Button as ReakitButton, ButtonProps as ReakitButtonProps, useButton as useReakitButton } from 'reakit';
import ConditionalWrap from 'conditional-wrap';

import { useClassName, createComponent, createElement, createHook } from '../utils';
import { ButtonType, Omit, Size, Palette } from '../types';
import { Box, BoxProps } from '../Box';
import { Flex } from '../Flex';
import { Icon, IconProps } from '../Icon';
import { Text } from '../Text';
import { Spinner, SpinnerProps } from '../Spinner';

import * as styles from './styles';

export type LocalButtonProps = {
  /** Icon that appears on the right side of the button. */
  iconAfter?: IconProps['icon'];
  iconAfterProps?: Omit<IconProps, 'icon'>;
  /** Icon that appears on the left side of the button. */
  iconBefore?: IconProps['icon'];
  iconBeforeProps?: Omit<IconProps, 'icon'>;
  /** Adds a loading indicator to the button. */
  isLoading?: boolean;
  /** Makes the button not interactable. */
  isStatic?: boolean;
  palette?: Palette;
  size?: Size;
  /** Custom props for the isLoading spinner. */
  spinnerProps?: SpinnerProps;
  type?: ButtonType;
};
export type ButtonProps = BoxProps & ReakitButtonProps & LocalButtonProps;

const useProps = createHook<ButtonProps>(
  (props, { themeKey, themeKeyOverride }) => {
    let {
      disabled,
      focusable,
      iconAfter,
      iconAfterProps,
      iconBefore,
      iconBeforeProps,
      spinnerProps,
      unstable_clickOnEnter,
      unstable_clickOnSpace,
      ...htmlProps
    } = props;
    const buttonProps = useReakitButton(
      { disabled, focusable, unstable_clickOnEnter, unstable_clickOnSpace },
      htmlProps
    );
    htmlProps = Box.useProps({ ...htmlProps, ...buttonProps });

    const className = useClassName({
      style: styles.Button,
      styleProps: props,
      themeKey,
      themeKeyOverride,
      prevClassName: htmlProps.className,
    });
    const iconBeforeClassName = useClassName({
      style: styles.ButtonIcon,
      styleProps: { ...props, isBefore: true },
      themeKey,
      themeKeyOverride,
      themeKeySuffix: 'Icon',
    });
    const iconAfterClassName = useClassName({
      style: styles.ButtonIcon,
      styleProps: { ...props, isAfter: true },
      themeKey,
      themeKeyOverride,
      themeKeySuffix: 'Icon',
    });
    const spinnerWrapperClassName = useClassName({
      style: styles.ButtonSpinnerWrapper,
      styleProps: props,
      themeKey,
      themeKeyOverride,
      themeKeySuffix: 'SpinnerWrapper',
    });
    const spinnerClassName = useClassName({
      style: styles.ButtonSpinner,
      styleProps: props,
      themeKey,
      themeKeyOverride,
      themeKeySuffix: 'Spinner',
    });

    const children = (
      <React.Fragment>
        {props.isLoading && (
          <Box className={spinnerWrapperClassName}>
            <Spinner
              use={Flex}
              className={spinnerClassName}
              color={props.variant === 'default' ? `${props.palette}Inverted` : props.palette}
              {...spinnerProps}
            />
          </Box>
        )}
        <ConditionalWrap condition={props.isLoading} wrap={(children) => <Text>{children}</Text>}>
          {iconBefore && <Icon className={iconBeforeClassName} icon={iconBefore} {...iconBeforeProps} />}
          {htmlProps.children}
          {iconAfter && <Icon className={iconAfterClassName} icon={iconAfter} {...iconAfterProps} />}
        </ConditionalWrap>
      </React.Fragment>
    );

    return { ...htmlProps, className, children };
  },
  {
    defaultProps: {
      disabled: false,
      iconAfter: undefined,
      iconBefore: undefined,
      isLoading: false,
      isStatic: false,
      variant: 'default',
      palette: 'default',
      size: 'default',
      type: 'button',
    },
    themeKey: 'Button',
  }
);

export const Button = createComponent<ButtonProps>(
  (props: ButtonProps) => {
    const buttonProps = useProps(props);
    return createElement({ children: props.children, component: ReakitButton, use: props.use, htmlProps: buttonProps });
  },
  {
    attach: {
      useProps,
      displayName: 'Button',
    },
    themeKey: 'Button',
  }
);
